==> Spirit-Aero-1.0.1-Horus.zip: Spirit-Aero-1.0.1/special.lua <==
-- Spirit Aero - Special Script
-- Copyright (C) 2016 Spirit System, FrSky dev@spirit-system.com
-- This file is distributed under Creative Commons 4 license - BY-NC-ND
-- Script version: 1.0.1

-- VALUE = 0
-- COMBO = 1
local BUTTON = 2
local MENU = 3
-- BLANK = 10
local LABEL = 20

local FA_INVIS = 1
local FA_BANK =  2

local edit = false
local page = 1
local current = 1
local refreshState = 0
local refreshIndex = 0
local pageOffset = 0
local pages = {}
local fields = {}
local modifications = {}
local selectCtr = 0
local value_tmp = 7
local chan_col = false

local profileFields = {
  {"SETTINGS", LABEL, FA_BANK },
  {"Save Settings", BUTTON, FA_BANK, 100, 0},
  { nil, 10, FA_BANK },
  { nil, 0, FA_BANK + FA_INVIS, 58, nil, 0, 7},
  {"BANK SWITCHING", LABEL, FA_BANK },
  {"Bank", 0, FA_BANK, 3, nil, 0, 2},
}

local governorFields = {
  {"Bank", 0, FA_INVIS, 3, nil, 0, 2},
  {"Rx", 0, FA_INVIS, 4, nil, 65, 70},
  {"Thr", 0, FA_INVIS, 52, nil, 0, 7},
  {"THROTTLE", LABEL, 0 },
  {"Range - Min.:", 0, 0, 42, nil, 70, 130, "0 us"},
  {"Range - Max.:", 0, 0, 43, nil, 170, 230, "0 us"},
  {"Reverse:", 1, 0, 41, nil, { "[ ]", "[x]" }, { 48, 49 } },
}

local channelsFields = {
  {"Bank", 0, FA_INVIS, 3, nil, 0, 2},
  {"Rx", 0, FA_INVIS, 4, nil, 65, 70},
  {"Throttle:", 1, 0, 52, nil, { "1", "2", "3", "4", "5", "6", "7", "unassigned" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Aileron:", 1, 0, 53, nil, { "1", "2", "3", "4", "5", "6", "7" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Elevator:", 1, 0, 54, nil, { "1", "2", "3", "4", "5", "6", "7" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Rudder:", 1, 0, 55, nil, { "1", "2", "3", "4", "5", "6", "7" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Global Gain:", 1, 0, 56, nil, { "1", "2", "3", "4", "5", "6", "7", "unassigned" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Flaps:", 1, 0, 57, nil, { "1", "2", "3", "4", "5", "6", "7", "unassigned" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Bank:", 1, 0, 58, nil, { "1", "2", "3", "4", "5", "6", "7", "unassigned" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  { nil, 10, 0 },
  {"Set Failsafe", BUTTON, 0, 102, 0},
}

local function addField(step)
  local field = fields[current]
  local min, max
  if field[2] == 0 then
    min = field[6]
    max = field[7]
  elseif field[2] == 1 then
    min = 0
    max = #(field[6]) - 1
  end
  if (step < 0 and field[5] > min) or (step > 0 and field[5] < max) then
    field[5] = field[5] + step
  end
end

local function selectPage(step)
  current = 1
  page = 1 + ((page + step - 1 + #pages) % #pages)
  refreshIndex = 0
  pageOffset = 0
end

local function selectField(step)
  if current <= #fields and fields[current][2] == BUTTON then
    fields[current][5] = 0
  end
  
  current = 1 + ((current + step - 1 + #fields) % #fields)
  
  local invisible = bit32.band(fields[current][3], FA_INVIS) ~= 0 

  if current > 10 + pageOffset then
    pageOffset = current - 10
  elseif current <= pageOffset then
    pageOffset = current - 1
  end
  
  if fields[current][2] == 10 or fields[current][2] == LABEL or invisible then
    selectCtr = selectCtr + 1
    
    if selectCtr < 16 then
      selectField (step)
    end
  end

  value_tmp = fields[current][5]
  --print ("v: " .. value_tmp)
  
  selectCtr = 0
end

local function drawProgressBar()
  local width = (140 * refreshIndex) / #fields
  lcd.drawRectangle(48+128, 11, 144, 6)
  lcd.drawFilledRectangle(50+128, 13, width, 2)
end

local function drawScreenTitle(title,page, pages)
  lcd.drawFilledRectangle(0, 0, LCD_W, 30, TITLE_BGCOLOR)
  lcd.drawText(1, 5, title, MENU_TITLE_COLOR)
  lcd.drawText(LCD_W-40, 5, page.."/"..pages, MENU_TITLE_COLOR)
end

local function redrawFieldsPage()
  lcd.clear()
  lcd.drawFilledRectangle(0, 0, LCD_W, LCD_H, TEXT_BGCOLOR)
  drawScreenTitle(pages[page][2], page, #pages)

  if refreshIndex < #fields then
    drawProgressBar()
  end

  local skip = 0
  local index = 0

  while true do
    index = index + 1
    if index - skip > 10 then
      break
    end
    local field = fields[pageOffset+index]
    if field == nil then
      break
    end
    
    local visible = bit32.band(field[3], FA_INVIS) == 0
    if not visible then
      skip = skip + 1
    end

    local attr = current == (pageOffset+index) and ((edit == true and BLINK or 0) + INVERS) or 0
    
    if current == (pageOffset+index) and (field[2] > MENU or not visible) then
      current = current + 1
    end
    
    if field[2] ~= 10 and visible then
      if field[2] == LABEL then
        lcd.drawText(2, 30+20*(index-skip), field[1] .. "", INVERS)
      else
        lcd.drawText(2, 30+20*(index-skip), " " .. field[1])
        if field[5] == nil then
          lcd.drawText(340, 30+20*(index-skip), "---", attr)
        else
          if field[2] == 0 then
            local val = field[5]
            if field[4] == 65 then
              val = (val * 100) / 20
              attr = attr + PREC2
            elseif field[4] == 20 then
              attr = attr + PREC2
            elseif field[4] == 73 then
              attr = attr + PREC1
            elseif field[4] == 34 then
              val = val * 10
            end
            if field[8] ~= nil then
              lcd.drawText(340, 30+20*(index-skip), val .. field[8], attr)
            else
              lcd.drawNumber(340, 30+20*(index-skip), val, LEFT + attr)
            end
          elseif field[2] == 1 then
            if field[5] >= 0 and field[5] < #(field[6]) then
              lcd.drawText(340, 30+20*(index-skip), field[6][1+field[5]], attr)
            end
          elseif field[2] == BUTTON or field[2] == MENU then
            local stat = "*"
            if field[5] == 1 then
              stat = "OK"
            end
            lcd.drawText(340, 30+20*(index-skip), stat, attr)
          end
        end
      end
    end
  end
  
  if index == (skip+1) then
    lcd.drawText(340/6, 30+20*4, "No parameter available. Bank: " .. fields[1][5], 0)
  end
end

local function telemetryRead(field)
  field = field + 0x81
  return sportTelemetryPush(0x1B, 0x30, 0x0C50, field)
end

local function telemetryWrite(field, value)
  field = field + 0x81
  return sportTelemetryPush(0x1B, 0x31, 0x0C50, field + value*256)
end

local function paramHandler(fieldId, value)
  if fieldId == 3 then
    for idx = 1, #fields, 1 do
      if (fields[idx][4] ~= 3 and fields[idx][4] ~= 4 and bit32.band(fields[idx][3], FA_BANK) == 0 and fields[idx][4] ~= 52) or (fields[idx][4] ~= 3 and fields[idx][4] ~= 4 and bit32.band(fields[idx][3], FA_BANK) == 0 and page == 2) then
        if value == 0 then
          fields[idx][3] = bit32.band(fields[idx][3], bit32.bnot(FA_INVIS))
        else
          fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
        end
      end
    end
  elseif fieldId == 4 and page ~= 2 then
    for idx = 9, #fields, 1 do
      if fields[idx][4] ~= 3 and fields[idx][4] ~= 4 then
        if value == 68 then
          if fields[1][5] == 0 then
            fields[idx][3] = bit32.band(fields[idx][3], bit32.bnot(FA_INVIS))
          end
        else
          fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
        end
      end
    end
    if value == 65 then
      for idx = 4, #fields, 1 do
        fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
      end
    end
  elseif fieldId == 4 and page == 2 and fields[1][5] == 0 then
    for idx = 3, #fields-2, 1 do
      if value ~= 65 and idx == 3 or idx == 7 or idx == 8 or idx == 9 then
        fields[idx][3] = bit32.band(fields[idx][3], bit32.bnot(FA_INVIS))
      else
        fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
      end
    end
  elseif fieldId == 52 and page ~= 2 then
    for idx = 1, #fields, 1 do
      if value == 7 then
        fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
      end
    end
  elseif fieldId == 58 and page ~= 2 then
    if value == 7 then
      fields[5][3] = bit32.bor(fields[5][3], FA_INVIS)
      fields[6][3] = bit32.bor(fields[6][3], FA_INVIS)
    else
      fields[5][3] = bit32.band(fields[5][3], bit32.bnot(FA_INVIS))
      fields[6][3] = bit32.band(fields[6][3], bit32.bnot(FA_INVIS))
    end
  elseif fieldId >= 52 and fieldId <= 58 and page == 2 then
    for idx = 3, #fields-2, 1 do
      if value == fields[idx][5] and value ~= 7 then
        if fields[idx][4] == fieldId then

        else
          chan_col = true
        end
      end
    end
  
    if fields[2][5] == 65 then
      if fieldId == 56 or fieldId == 58 then
        if value ~= 4 and value ~= 6 and value ~= 7 then
          chan_col = true
        end
      end
      if fieldId == 57 then
        if value ~= 5 and value ~= 7 then
          chan_col = true
        end
      end
    end
  end
end

local telemetryPopTimeout = 0
local function refreshNext()
  if refreshState == 0 then
    if #modifications > 0 then
      if modifications[1][1] >= 52 and modifications[1][1] <= 58 then
        paramHandler(modifications[1][1], modifications[1][2])
      end
      if chan_col == false then
        telemetryWrite(modifications[1][1], modifications[1][2])
      else
        chan_col = false
        fields[current][5] = value_tmp
      end
      if modifications[1][1] == 30 then
        paramHandler(modifications[1][1], modifications[1][2])
        refreshIndex = 4
      end
      if page == 2 and fields[2][5] == 65 then
        if modifications[1][2] == 6 then
          refreshIndex = current-1
        end
      end
      modifications[1] = nil
    elseif refreshIndex < #fields then
      local field = fields[refreshIndex + 1]
      if field[4] ~= nil and field[2] ~= BUTTON and field[2] ~= MENU and bit32.band(field[3], FA_INVIS) == 0 or field[4] == 3 or field[4] == 4 or field[4] == 52 or field[4] == 53 or field[4] == 54 or field[4] == 55 or field[4] == 56 or field[4] == 57 or field[4] == 58 then
        if telemetryRead(field[4]) == true then
          refreshState = 1
          telemetryPopTimeout = getTime() + 80
        end
      else
        refreshIndex = refreshIndex + 1 
      end
    end
  elseif refreshState == 1 then
    local physicalId, primId, dataId, value = sportTelemetryPop()
    if physicalId == 0x1A and primId == 0x32 and dataId == 0x0C50 then
      local fieldId = (value % 256) - 0x81
      local field = fields[refreshIndex + 1]
      if fieldId == field[4] then
        local value = math.floor(value / 256)
        paramHandler(fieldId, value)
        
        if field[2] == 1 and #field == 7 then
          for index = 1, #(field[7]), 1 do
            if value == field[7][index] then
              value = index - 1
              break
            end
          end
        elseif field[2] == 0 and #field == 9 then
          value = value + field[9]
        end
        fields[refreshIndex + 1][5] = value
        
        refreshIndex = refreshIndex + 1
        refreshState = 0
      end
    elseif getTime() > telemetryPopTimeout then
      refreshState = 0
    end
  end
end

local function updateField(field)
  local value = field[5]
  if field[2] == 1 and #field == 7 then
    value = field[7][1+value]
  elseif field[2] == 0 and #field == 9 then
    value = value - field[9]
  elseif field[2] == BUTTON then
    fields[current][5] = 1
    value = 1
  end
  modifications[#modifications+1] = { field[4], value }
end

local function runFieldsPage(event)
  if event == EVT_EXIT_BREAK then
    telemetryWrite(101, 0)
    return 2
  elseif event == EVT_ENTER_BREAK then
    if fields[current] ~= nil then
      if fields[current][2] == BUTTON then
        if fields[current][5] == 0 then
          updateField(fields[current])
        end
      elseif fields[current][2] == MENU then
        local tmp = pages[page][1]
        local menu = 1+fields[current][5]
        
        pages[page][1] = pages[page][3][menu]
        pages[page][3][menu] = tmp

        selectPage (0)
      elseif fields[current][5] ~= nil then
        edit = not edit
        if edit == false then
          updateField(fields[current])
        end
      end
    end
  elseif edit then
    if event == EVT_PLUS_FIRST or event == EVT_PLUS_REPT or event == EVT_ROT_RIGHT then
      addField(1)
    elseif event == EVT_MINUS_FIRST or event == EVT_MINUS_REPT or event == EVT_ROT_LEFT then
      addField(-1)
    end
  else
    if event == EVT_MINUS_FIRST or event == EVT_ROT_RIGHT then
      selectField(1)
    elseif event == EVT_PLUS_FIRST or event == EVT_ROT_LEFT then
      selectField(-1)
    end
  end
  redrawFieldsPage()
  return 0
end

local function init()
  current, edit, refreshState, refreshIndex = 1, false, 0, 0
  pages = {
    {governorFields, "Throttle"},
    {channelsFields, "Channels"},
    {profileFields, "Profile"},
  }
end

local function run(event)
  if event == nil then
    return 2
  elseif event == EVT_PAGE_BREAK or event == EVT_PAGEDN_FIRST then
    selectPage(1)
  elseif event == EVT_PAGE_LONG or event == EVT_PAGEUP_FIRST then
    killEvents(event);
    selectPage(-1)
  end
  fields = pages[page][1]
  local result = runFieldsPage(event)
  refreshNext()
  return result
end

return { init=init, run=run }

==> Spirit-Aero-1.1.0-Horus.zip: Spirit-Aero-1.1.0/special.lua <==
-- Spirit Aero - Special Script
-- Copyright (C) 2017 Spirit System, FrSky dev@spirit-system.com
-- This file is distributed under Creative Commons 4 license - BY-NC-ND
-- Script version: 1.1.0

-- VALUE = 0
-- COMBO = 1
local BUTTON = 2
local MENU = 3
-- BLANK = 10
local LABEL = 20

local FA_INVIS = 1
local FA_BANK =  2

local edit = false
local page = 1
local current = 1
local refreshState = 0
local refreshIndex = 0
local pageOffset = 0
local pages = {}
local fields = {}
local modifications = {}
local selectCtr = 0
local value_tmp = 7
local chan_col = false

local profileFields = {
  {"SETTINGS", LABEL, FA_BANK },
  {"Save Settings", BUTTON, FA_BANK, 100, 0},
  { nil, 10, FA_BANK },
  { nil, 0, FA_BANK + FA_INVIS, 58, nil, 0, 7},
  {"BANK SWITCHING", LABEL, FA_BANK },
  {"Bank", 0, FA_BANK, 3, nil, 0, 2},
}

local governorFields = {
  {"Bank", 0, FA_INVIS, 3, nil, 0, 2},
  {"Rx", 0, FA_INVIS, 4, nil, 65, 70},
  {"Thr", 0, FA_INVIS, 52, nil, 0, 7},
  {"THROTTLE", LABEL, 0 },
  {"Range - Min.:", 0, 0, 42, nil, 70, 130, "0 us"},
  {"Range - Max.:", 0, 0, 43, nil, 170, 230, "0 us"},
  {"Reverse:", 1, 0, 41, nil, { "[ ]", "[x]" }, { 48, 49 } },
}

local channelsFields = {
  {"Bank", 0, FA_INVIS, 3, nil, 0, 2},
  {"Rx", 0, FA_INVIS, 4, nil, 65, 70},
  {"Throttle:", 1, 0, 52, nil, { "1", "2", "3", "4", "5", "6", "7", "unassigned" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Aileron:", 1, 0, 53, nil, { "1", "2", "3", "4", "5", "6", "7" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Elevator:", 1, 0, 54, nil, { "1", "2", "3", "4", "5", "6", "7" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Rudder:", 1, 0, 55, nil, { "1", "2", "3", "4", "5", "6", "7" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Global Gain:", 1, 0, 56, nil, { "1", "2", "3", "4", "5", "6", "7", "unassigned" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Flaps:", 1, 0, 57, nil, { "1", "2", "3", "4", "5", "6", "7", "unassigned" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  {"Bank:", 1, 0, 58, nil, { "1", "2", "3", "4", "5", "6", "7", "unassigned" }, { 0, 1, 2, 3, 4, 5, 6, 7 } },
  { nil, 10, 0 },
  {"Set Failsafe", BUTTON, 0, 102, 0},
}

local function addField(step)
  local field = fields[current]
  local min, max
  if field[2] == 0 then
    min = field[6]
    max = field[7]
  elseif field[2] == 1 then
    min = 0
    max = #(field[6]) - 1
  end
  if (step < 0 and field[5] > min) or (step > 0 and field[5] < max) then
    field[5] = field[5] + step
  end
end

local function selectPage(step)
  current = 1
  page = 1 + ((page + step - 1 + #pages) % #pages)
  refreshIndex = 0
  pageOffset = 0
end

local function selectField(step)
  if current <= #fields and fields[current][2] == BUTTON then
    fields[current][5] = 0
  end
  
  current = 1 + ((current + step - 1 + #fields) % #fields)
  
  local invisible = bit32.band(fields[current][3], FA_INVIS) ~= 0 

  if current > 10 + pageOffset then
    pageOffset = current - 10
  elseif current <= pageOffset then
    pageOffset = current - 1
  end
  
  if fields[current][2] == 10 or fields[current][2] == LABEL or invisible then
    selectCtr = selectCtr + 1
    
    if selectCtr < 16 then
      selectField (step)
    end
  end

  value_tmp = fields[current][5]
  --print ("v: " .. value_tmp)
  
  selectCtr = 0
end

local function drawProgressBar()
  local width = (140 * refreshIndex) / #fields
  lcd.drawRectangle(48+128, 11, 144, 6)
  lcd.drawFilledRectangle(50+128, 13, width, 2)
end

local function drawScreenTitle(title,page, pages)
  lcd.drawFilledRectangle(0, 0, LCD_W, 30, TITLE_BGCOLOR)
  lcd.drawText(1, 5, title, MENU_TITLE_COLOR)
  lcd.drawText(LCD_W-40, 5, page.."/"..pages, MENU_TITLE_COLOR)
end

local function redrawFieldsPage()
  lcd.clear()
  lcd.drawFilledRectangle(0, 0, LCD_W, LCD_H, TEXT_BGCOLOR)
  drawScreenTitle(pages[page][2], page, #pages)

  if refreshIndex < #fields then
    drawProgressBar()
  end

  local skip = 0
  local index = 0

  while true do
    index = index + 1
    if index - skip > 10 then
      break
    end
    local field = fields[pageOffset+index]
    if field == nil then
      break
    end
    
    local visible = bit32.band(field[3], FA_INVIS) == 0
    if not visible then
      skip = skip + 1
    end

    local attr = current == (pageOffset+index) and ((edit == true and BLINK or 0) + INVERS) or 0
    
    if current == (pageOffset+index) and (field[2] > MENU or not visible) then
      current = current + 1
    end
    
    if field[2] ~= 10 and visible then
      if field[2] == LABEL then
        lcd.drawText(2, 30+20*(index-skip), field[1] .. "", INVERS)
      else
        
        lcd.drawText(2, 30+20*(index-skip), " " .. field[1])
        if field[5] == nil then
          lcd.drawText(340, 30+20*(index-skip), "---", attr)
        else
          if field[2] == 0 then
            local val = field[5]
            if field[4] == 65 then
              val = (val * 100) / 20
              attr = attr + PREC2
            elseif field[4] == 20 then
              attr = attr + PREC2
            elseif field[4] == 73 then
              attr = attr + PREC1
            elseif field[4] == 34 then
              val = val * 10
            end
            if field[8] ~= nil then
              lcd.drawText(340 - string.len(val .. field[8]), 30+20*(index-skip), val .. field[8], attr)
            else
              lcd.drawNumber(340 - string.len(val), 30+20*(index-skip), val, LEFT + attr)
            end
          elseif field[2] == 1 then
            if field[5] >= 0 and field[5] < #(field[6]) then
              lcd.drawText(340 - string.len(field[6][1+field[5]]), 30+20*(index-skip), field[6][1+field[5]], attr)
            end
          elseif field[2] == BUTTON or field[2] == MENU then
            local stat = "*"
            if field[5] == 1 then
              stat = "OK"
            end
            lcd.drawText(340 - string.len(stat), 30+20*(index-skip), stat, attr)
          end
        end
      end
    end
  end
  
  if index == (skip+1) then
    lcd.drawText(340/6, 30+20*4, "No parameter available. Bank: " .. fields[1][5], 0)
  end
end

local function telemetryRead(field)
  field = field + 0x81
  return sportTelemetryPush(0x1B, 0x30, 0x0C50, field)
end

local function telemetryWrite(field, value)
  field = field + 0x81
  return sportTelemetryPush(0x1B, 0x31, 0x0C50, field + value*256)
end

local function paramHandler(fieldId, value)
  if fieldId == 3 then
    for idx = 1, #fields, 1 do
      if (fields[idx][4] ~= 3 and fields[idx][4] ~= 4 and bit32.band(fields[idx][3], FA_BANK) == 0 and fields[idx][4] ~= 52) or (fields[idx][4] ~= 3 and fields[idx][4] ~= 4 and bit32.band(fields[idx][3], FA_BANK) == 0 and page == 2) then
        if value == 0 then
          fields[idx][3] = bit32.band(fields[idx][3], bit32.bnot(FA_INVIS))
        else
          fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
        end
      end
    end
  elseif fieldId == 4 and page ~= 2 then
    for idx = 9, #fields, 1 do
      if fields[idx][4] ~= 3 and fields[idx][4] ~= 4 then
        if value == 68 then
          if fields[1][5] == 0 then
            fields[idx][3] = bit32.band(fields[idx][3], bit32.bnot(FA_INVIS))
          end
        else
          fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
        end
      end
    end
    if value == 65 then
      for idx = 4, #fields, 1 do
        fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
      end
    end
  elseif fieldId == 4 and page == 2 and fields[1][5] == 0 then
    for idx = 3, #fields-2, 1 do
      if value ~= 65 and idx == 3 or idx == 7 or idx == 8 or idx == 9 then
        fields[idx][3] = bit32.band(fields[idx][3], bit32.bnot(FA_INVIS))
      else
        fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
      end
    end
  elseif fieldId == 52 and page ~= 2 then
    for idx = 1, #fields, 1 do
      if value == 7 then
        fields[idx][3] = bit32.bor(fields[idx][3], FA_INVIS)
      end
    end
  elseif fieldId == 58 and page ~= 2 then
    if value == 7 then
      fields[5][3] = bit32.bor(fields[5][3], FA_INVIS)
      fields[6][3] = bit32.bor(fields[6][3], FA_INVIS)
    else
      fields[5][3] = bit32.band(fields[5][3], bit32.bnot(FA_INVIS))
      fields[6][3] = bit32.band(fields[6][3], bit32.bnot(FA_INVIS))
    end
  elseif fieldId >= 52 and fieldId <= 58 and page == 2 then
    for idx = 3, #fields-2, 1 do
      if value == fields[idx][5] and value ~= 7 then
        if fields[idx][4] == fieldId then

        else
          chan_col = true
        end
      end
    end
  
    if fields[2][5] == 65 then
      if fieldId == 56 or fieldId == 58 then
        if value ~= 4 and value ~= 6 and value ~= 7 then
          chan_col = true
        end
      end
      if fieldId == 57 then
        if value ~= 5 and value ~= 7 then
          chan_col = true
        end
      end
    end
  end
end

local telemetryPopTimeout = 0
local function refreshNext()
  if refreshState == 0 then
    if #modifications > 0 then
      if modifications[1][1] >= 52 and modifications[1][1] <= 58 then
        paramHandler(modifications[1][1], modifications[1][2])
      end
      if chan_col == false then
        telemetryWrite(modifications[1][1], modifications[1][2])
      else
        chan_col = false
        fields[current][5] = value_tmp
      end
      if modifications[1][1] == 30 then
        paramHandler(modifications[1][1], modifications[1][2])
        refreshIndex = 4
      end
      if page == 2 and fields[2][5] == 65 then
        if modifications[1][2] == 6 then
          refreshIndex = current-1
        end
      end
      modifications[1] = nil
    elseif refreshIndex < #fields then
      local field = fields[refreshIndex + 1]
      if field[4] ~= nil and field[2] ~= BUTTON and field[2] ~= MENU and bit32.band(field[3], FA_INVIS) == 0 or field[4] == 3 or field[4] == 4 or field[4] == 52 or field[4] == 53 or field[4] == 54 or field[4] == 55 or field[4] == 56 or field[4] == 57 or field[4] == 58 then
        if telemetryRead(field[4]) == true then
          refreshState = 1
          telemetryPopTimeout = getTime() + 80
        end
      else
        refreshIndex = refreshIndex + 1 
      end
    end
  elseif refreshState == 1 then
    local physicalId, primId, dataId, value = sportTelemetryPop()
    if physicalId == 0x1A and primId == 0x32 and dataId == 0x0C50 then
      local fieldId = (value % 256) - 0x81
      local field = fields[refreshIndex + 1]
      if fieldId == field[4] then
        local value = math.floor(value / 256)
        paramHandler(fieldId, value)
        
        if field[2] == 1 and #field == 7 then
          for index = 1, #(field[7]), 1 do
            if value == field[7][index] then
              value = index - 1
              break
            end
          end
        elseif field[2] == 0 and #field == 9 then
          value = value + field[9]
        end
        fields[refreshIndex + 1][5] = value
        
        refreshIndex = refreshIndex + 1
        refreshState = 0
      end
    elseif getTime() > telemetryPopTimeout then
      refreshState = 0
    end
  end
end

local function updateField(field)
  local value = field[5]
  if field[2] == 1 and #field == 7 then
    value = field[7][1+value]
  elseif field[2] == 0 and #field == 9 then
    value = value - field[9]
  elseif field[2] == BUTTON then
    fields[current][5] = 1
    value = 1
  end
  modifications[#modifications+1] = { field[4], value }
end

local function runFieldsPage(event)
  if event == EVT_EXIT_BREAK then
    telemetryWrite(101, 0)
    return 2
  elseif event == EVT_ENTER_BREAK then
    if fields[current] ~= nil then
      if fields[current][2] == BUTTON then
        if fields[current][5] == 0 then
          updateField(fields[current])
        end
      elseif fields[current][2] == MENU then
        local tmp = pages[page][1]
        local menu = 1+fields[current][5]
        
        pages[page][1] = pages[page][3][menu]
        pages[page][3][menu] = tmp

        selectPage (0)
      elseif fields[current][5] ~= nil then
        edit = not edit
        if edit == false then
          updateField(fields[current])
        end
      end
    end
  elseif edit then
    if event == EVT_PLUS_FIRST or event == EVT_PLUS_REPT or event == EVT_ROT_RIGHT then
      addField(1)
    elseif event == EVT_MINUS_FIRST or event == EVT_MINUS_REPT or event == EVT_ROT_LEFT then
      addField(-1)
    end
  else
    if event == EVT_MINUS_FIRST or event == EVT_ROT_RIGHT then
      selectField(1)
    elseif event == EVT_PLUS_FIRST or event == EVT_ROT_LEFT then
      selectField(-1)
    end
  end
  redrawFieldsPage()
  return 0
end

local function init()
  current, edit, refreshState, refreshIndex = 1, false, 0, 0
  pages = {
    {governorFields, "Throttle"},
    {channelsFields, "Channels"},
    {profileFields, "Profile"},
  }
end

local function run(event)
  if event == nil then
    return 2
  elseif event == EVT_PAGE_BREAK or event == EVT_PAGEDN_FIRST then
    selectPage(1)
  elseif event == EVT_PAGE_LONG or event == EVT_PAGEUP_FIRST then
    killEvents(event);
    selectPage(-1)
  end
  fields = pages[page][1]
  local result = runFieldsPage(event)
  refreshNext()
  return result
end

return { init=init, run=run }
